function [chFRhex] = GetChRF(liveflag,ch, freq, amp)
%Returnes G&H 16 channel hex for channel configuration
% INPUT:
%   liveflag  - 0=live update, 1=profiles upload
%   ch        - channel number from 0 to 15
%   freq      - frequency in MHz
%   amp       - amplitude in % max
% OUTPUT:
%   chFRhex   - structure with corresponding hexes

if(liveflag==0)
    fb = '05'; %command hex byte for immediate execution
else
    fb = '06'; %command hex byte for profile load
end

% set channel
chHex = getChHex(ch);
chFRhex(1) = {[fb,chHex,'000000']};

% set frequency
chFHex = getChFHex(ch);
f14 = dec2hex(int32(2^32*freq/500));
if(freq==0)
    f14='00000000';
end
chFRhex(2) = {[fb,chFHex,f14]};

% set phase - first two bytes sets the phase, here set to 0
chPHex = getChPHex(ch);
chFRhex(3) = {[fb,chPHex,'00000000']};

% set amplitude
chAHex = getChAHex(ch);
coeff = 10.23;
if(ch == 1 || ch == 5 || ch == 9 || ch == 13)
    coeff = 2.55;
end
a12 = dec2hex(int16((amp*coeff)+4096));%multiplication by coeff might be wrong
chFRhex(4) = {[fb,chAHex,'00',a12,'00']};

%change to hexMatrix
s = chFRhex;
chFRhex = [];
for i=1:4
    is = s{i};
    a=[];
    for j=1:6
        a = [a;is(2*j-1:2*j)];
    end
    chFRhex{i}=a;
end
end

%function returns Hex for a channel
function chHex = getChHex(ch)
    switch(ch)
        case 0
            chHex = '0016';
        case 1
            chHex = '0026';
        case 2
            chHex = '0046';
        case 3
            chHex = '0086';
        case 4
            chHex = '2016';
        case 5
            chHex = '2026';
        case 6
            chHex = '2046';
        case 7
            chHex = '2086';
        case 8
            chHex = '4016';
        case 9
            chHex = '4026';
        case 10
            chHex = '4046';
        case 11
            chHex = '4086';
        case 12
            chHex = '6016';
        case 13
            chHex = '6026';
        case 14
            chHex = '6046';
        case 15
            chHex = '6086';            
    end
end
% frequency Hex
function chFHex = getChFHex(ch)    
    if(ch<4)
    	chFHex = '04';
    elseif(ch>3 && ch<8)
        chFHex = '24';
    elseif(ch>7 && ch<12)
        chFHex = '44';
    elseif(ch>11)
        chFHex = '64';
    end
end

% phase Hex
function chPHex = getChPHex(ch)
    if(ch<4)
    	chPHex = '05';
    elseif(ch>3 && ch<8)
        chPHex = '25';
    elseif(ch>7 && ch<12)
        chPHex = '45';
    elseif(ch>11)
        chPHex = '65';
    end
end

% amplitude Hex
function chAHex = getChAHex(ch)
    if(ch<4)
    	chAHex = '06';
    elseif(ch>3 && ch<8)
        chAHex = '26';
    elseif(ch>7 && ch<12)
        chAHex = '46';
    elseif(ch>11)
        chAHex = '66';
    end
end