%Load data
info = readHyperHeader('Grey450-808.hdr');
dark = multibandread('Dark450-808.bip', [info.lines,info.samples,info.bands], info.data_type, 0, info.interleave, 'ieee-le');
gray = multibandread('Grey450-808.bip', [info.lines,info.samples,info.bands], info.data_type, 0, info.interleave, 'ieee-le');
img = multibandread('Grid450-808.bip', [info.lines,info.samples,info.bands], info.data_type, 0, info.interleave, 'ieee-le');
[imgD] = SubDark(img,dark);
[grayD] = SubDark(gray,dark);

%Stract wavelengths vector
lam = zeros(1,info.bands);
a=1;
for i=2:8:1439
    lam(a)=str2double(info.band_names(i:i+2));
    a = a+1;
end

%Reflectance image
imgR = img2R(imgD,grayD, lam);

%Data set that will be modified
ModifiedGrid = zeros(1000, 1000, 180);
for i=1:180;
    ModifiedGrid(:,:,i) = imgR(i,:,:);
end

% Grid draw: maximum value
range = [-6 : 6];
columna = [30 91 122 248 375 502 661 789 881 974];
verticalDistortion = zeros(180, length(columna));
for l=1:length(columna)
    for k = 1:180;
        for i=50:870
            ref = imgR(k, i, columna(l)+range);
            [~, I] = min(ref);
            offset = range(I);
            verticalDistortion(k, l) = verticalDistortion(k, l) + abs(offset)^2;
            ModifiedGrid(i, columna(l)+offset, k)=0;
        end
    end
end

row = [108 203 329 424 457 520 615 710 804];
horizontalDistortion = zeros(180, length(row));
for l=1:length(row)
    for k = 1:180;
        for i=1:1000
            ref = imgR(k, row(l)+range, i);        
            [~, I] = min(ref);            
            offset = range(I);
            horizontalDistortion(k, l) = horizontalDistortion(k, l) + abs(offset)^2;
            ModifiedGrid(row(l)+offset, i, k)=0;
        end
    end
end

% Grid draw: straight lines
ModifiedGrid(:, columna, :) = 0;
ModifiedGrid(row, :, :) = 0;

% Normalize vertical and horizontal distortion: 1 means no distortion
Normalization = min([min(min(verticalDistortion)) min(min(horizontalDistortion))]);
    verticalDistortion = verticalDistortion ./ Normalization;
    horizontalDistortion = horizontalDistortion ./ Normalization;

% Plot distortion for different wavelengths
color = [0 0 255; 102 178 255;...
         0 255 0; 102 255 178;...
         255 0 0; 255 178 102]/255;
t=1;
s=1;
for h = 10:30:180
plot(columna, verticalDistortion(h, :), 'Color', color(t,:));
strings{s} = [num2str(lam(h)) 'nm'];
hold on;
t = t+1; s = s+1;
end
plot(columna, ones(length(columna)), 'Color', [153 153 255]/255,'LineStyle', '--')
hold on;
plot([499.99:0.01:500.01],[0 20 0],  'Color', [255 153 204]/255,'LineStyle', ':')
hold on;
xlabel('Pixel');
ylabel('Vertical distortion');
strings{s}='No distortion';
strings{s+1}='Center';
legend(strings)
axis([0 1000 0 20])

figure(2);
t=1;
s=1;
for h = 10:30:180
plot(row, horizontalDistortion(h, :), 'Color', color(t,:));
strings{s} = [num2str(lam(h)) 'nm'];
t = t+1;s = s+1;
hold on;
end
plot(row, ones(length(row)), 'Color', [153 153 255]/255,'LineStyle', '--')
hold on;
plot([456.99:0.01:457.01],[0 20 0],  'Color', [255 153 204]/255,'LineStyle', ':')
hold on;
xlabel('Pixel');
ylabel('Horizontal distortion');
strings{s}='No distortion';
strings{s+1}='Center';
legend(strings)
axis([0 1000 0 20])

% Calculate total distortion and plot it
averageVert = zeros(1, 180);
for i=1:180
averageVert(i) = sum(verticalDistortion(i,:))/length(verticalDistortion(1,:));
end
averageHor = zeros(1, 180);
for i=1:180
averageHor(i) = sum(horizontalDistortion(i,:))/length(horizontalDistortion(1,:));
end

smoothedVert = averageVert;
smoothedHor = averageHor;
for i=3:177
    smoothedVert(i) = 1/5*(averageVert(i-2) + averageVert(i-1) + averageVert(i) +...
        averageVert(i+2) + averageVert(i+1));
    smoothedHor(i) = 1/5*(averageHor(i-2) + averageHor(i-1) + averageHor(i) +...
        averageHor(i+2) + averageHor(i+1));
end
figure(3);
plot(lam, smoothedVert);
hold on;
plot(lam, smoothedHor);
xlabel('Wavelength');
ylabel('Distortion');
strings = {'Vertical', 'Horizontal'};
legend(strings);

figure(4);
columna6 = verticalDistortion(:, 6)
N = 3;
n = floor(N/2)
n_vect = [-n:n];
for i=1+n:180-n
    columna6(i) = 1/3*(sum(columna6(i+n_vect)))
end
plot(lam, columna6)
xlabel('Wavelength');
ylabel('Distortion central column');


