/*********************************************************************
* We get a frame, the AOTF should be already set up
* 
* [imagen] = mGetFrameLiveView(double handle, double dimensions);

dimensions = [pix_height, pix_width, B_ImageSize, B_stride, success, transposed]

* Keep in mind (for all mex files):
* <> Use 0-based indexing as always in C or C++
* <> Indexing is column-based as in Matlab (not row-based as in C)
* <> Use linear indexing.  [x*dimy+y] instead of [x][y]
********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include "atcore.h"
#include <iostream> 
#include "stdlib.h"
#include <time.h>
#include <windows.h>

using namespace std;

/*Define uint64 for convenience*/
typedef unsigned long long uint64;

void mexFunction(int nlhs, mxArray *plhs[], 
	int nrhs, const mxArray *prhs[])
{
	mxArray *image_out; 
	double *handle;
	unsigned char *image;
	unsigned short *uint16_image;
	mwSize *dimensions;

	int i_retCode;
	AT_64 height, width, stride, ImageSizeBytes;

    /* Count the time it takes to execute this function, useful
    * to improve it and analise its performance*/
	clock_t getImage = clock();

	/*Get inputs*/
	handle = mxGetPr(prhs[0]);
    dimensions = static_cast<mwSize *> (mxGetData(prhs[1]));
    
    /*Hndl must be AT_H when passed to camera*/
	AT_H Hndl;
	Hndl = static_cast<AT_H> (*handle);	
    
	/*Allocate memory */
	int imageSizeBytes;
	imageSizeBytes = dimensions[2];
	image = static_cast<unsigned char *>(mxMalloc((imageSizeBytes+8)*sizeof(unsigned char)));	
	
    /*Align the image in memory, usually not necessary: compiler does it*/
    //image = reinterpret_cast<unsigned char*>( (reinterpret_cast<unsigned long>( image ) + 7 ) & ~0x7);

	/*Queue a buffer (image) and send a SoftwareTrigger to get an image*/	
	i_retCode = AT_QueueBuffer(Hndl, image, dimensions[2]);

	AT_BOOL acqui;
	i_retCode =	AT_GetBool(Hndl, L"CameraAcquiring", &acqui);	

	i_retCode = AT_Command(Hndl, L"SoftwareTrigger");			

    /* Receive the last buffer available*/
	unsigned char* WaitBuffer;
	int i_waitBufferSize;
	i_retCode = AT_WaitBuffer(Hndl, &WaitBuffer, &i_waitBufferSize, 3000);		

    /* Check that its address is the same as that of image*/
	if ( WaitBuffer != image ) {	
		mexPrintf("WARNING!!");
	}		

	/*OUTPUT*/
	
    /*Image: first convert the buffer from char to unsigned short*/
	uint16_image = reinterpret_cast<unsigned short*>(image);
	plhs[0] = mxCreateNumericMatrix(0, 0, mxUINT16_CLASS, mxREAL);			
	image_out = plhs[0];
    
    /* Assign the output*/
	mxSetData(image_out, uint16_image);
	mwSize row;
	row = dimensions[0];
	//row+=2;
	mxSetM(plhs[0], dimensions[1]);
	mxSetN(plhs[0], row);
		
    /*Calculate the time that took the function execution and print it*/
	mexPrintf("\nTime getImage: %.9fs\n", (float)(clock() - getImage)/CLOCKS_PER_SEC);
	return;
}
