/*********************************************************************
* [dimensions, HSI, timestamp] = mGetHSISoftware(double handle, double *wavelengths);

dimensions = [Nlam, pix_height, pix_width, B_ImageSize, B_stride, success, transposed]
HSI is the pix_height*pix_width*Nlam
*
*
* Keep in mind:
* <> Use 0-based indexing as always in C or C++
* <> Indexing is column-based as in Matlab (not row-based as in C)
* <> Use linear indexing.  [x*dimy+y] instead of [x][y]
********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include "atcore.h"
#include <iostream> 
#include "stdlib.h"
#include <windows.h>

#include <time.h>

//#include "initAOTF.cpp"

using namespace std;

typedef unsigned long long uint64;

void getImage(unsigned char *HSI, mwSize *dimensions, double handle, int *wavelengths);
uint64 extractMetadata(unsigned char* metadata, int length_metadata);

void mexFunction(int nlhs, mxArray *plhs[], 
	int nrhs, const mxArray *prhs[])
{
	mxArray  *dimensions_out, *HSI_out, *timestamp_out; 
	double *handle;
	unsigned char *HSI;
	unsigned short *uint16_HSI;
	mwSize *dimensions;
	int *wavelengths, Nlam;

	int i_retCode;   
	AT_64 height, width, stride, ImageSizeBytes;

	/*Get inputs*/
	handle = mxGetPr(prhs[0]);

	AT_H Hndl;
	Hndl = static_cast<AT_H> (*handle);	

	wavelengths = static_cast<int *> (mxGetData(prhs[1]));	
	Nlam = static_cast<int> (mxGetN(prhs[1]));
    Nlam = Nlam + 1;


	/*Allocate memory for dimensions*/
	dimensions = static_cast<mwSize *> (mxMalloc(7 * sizeof(mwSize)));	

	/*Get image dimensions, it could be easily edited to set dimensions
     instead of get them, and thus get customed sized images*/
	i_retCode = AT_GetInt(Hndl, L"AOIHeight", &height);
	i_retCode = AT_GetInt(Hndl, L"AOIWidth", &width);
	i_retCode = AT_GetInt(Hndl, L"ImageSizeBytes", &ImageSizeBytes);
	i_retCode = AT_GetInt(Hndl, L"AOIStride", &stride);
	dimensions[0] = static_cast<mwSize> (Nlam);
	dimensions[1] = static_cast<mwSize> (height);	
	dimensions[2] = static_cast<mwSize> (width);	
	int i_imageSize = static_cast<int>(ImageSizeBytes);
	dimensions[3] = static_cast<mwSize> (i_imageSize);
	dimensions[4] = static_cast<mwSize> (stride);

	/*Allocate memory and align the image*/
	HSI = static_cast<unsigned char *> (mxMalloc((Nlam  * i_imageSize + 8) * sizeof(unsigned char)));	
	//HSI = reinterpret_cast<unsigned char *>( (reinterpret_cast<unsigned long>( HSI ) + 7 ) & ~0x7);


	/*Initialize AOTF*/
//	i_retCode = initAOTF(50);

	/*We get the image*/
	getImage(HSI, dimensions, *handle, wavelengths);
	

	if (dimensions[5] == 1){ 
        
        /*Image was succesfully acquired*/
		uint64 *timestamp;
		timestamp = static_cast<uint64 *> (mxMalloc(Nlam * sizeof(uint64)));

		/*Stract the metadata string*/
		int fin, init, len;
		unsigned char *metadata;	
		init =  2*dimensions[1]*dimensions[2];
		fin = dimensions[3];
		len = fin-init;

		metadata = new unsigned char[len];
		
		int offset=0;
		for (int k=0; k < Nlam; k++){
			offset =  k * dimensions[3];
			for(int i=init; i<fin; i++){
				metadata[i-init] = HSI[i+offset];
			}
            /*Get the timestamp from the Metadata string*/
			timestamp[k] = extractMetadata(metadata, len);
		}

		/*Set the transposed flag*/
		dimensions[6] = 1;

		/*OUTPUT*/
        
		/*Dimensions*/
		plhs[0] = mxCreateNumericMatrix(1, 7, mxUINT32_CLASS, mxREAL);	
		dimensions_out = plhs[0];
		mxSetData(dimensions_out, dimensions);

		/*Image*/
		uint16_HSI = reinterpret_cast<unsigned short*>(HSI);

		const mwSize dims_zero[3] = {0, 0, 0};
		plhs[1] = mxCreateNumericArray(3, dims_zero, mxUINT16_CLASS, mxREAL);	
		 
		HSI_out = plhs[1];

		mxSetData(HSI_out, uint16_HSI);

		mwSize dims[3];
		dims[2] = Nlam;
		dims[0] = dimensions[2];
		dims[1] = dimensions[1]+2; //because of the metadata

		i_retCode = mxSetDimensions(plhs[1], dims, 3);

		/*Timestamp*/
		plhs[2] = mxCreateNumericMatrix(1, Nlam, mxUINT64_CLASS, mxREAL);	
		timestamp_out = plhs[2];
		mxSetData(timestamp_out, timestamp);
		mxSetM(plhs[2], 1);
		mxSetN(plhs[2], Nlam);

		/*Free allocated memory*/
		delete metadata;
	}
	else{		

		/*OUTPUT in case image acquisition fails*/

		/*Dimensions, in the flag dimensions[5] we have the notification that something failed*/
		plhs[0] = mxCreateNumericMatrix(1, 7, mxUINT32_CLASS, mxREAL);	
		dimensions_out = plhs[0];
		mxSetData(dimensions_out, dimensions);

		/*Image just a 0*/
		plhs[1] = mxCreateNumericMatrix(1, 1, mxUINT16_CLASS, mxREAL);			
		HSI_out = plhs[1];
		uint16_HSI = static_cast<unsigned short *> (mxMalloc(1*sizeof(unsigned short)));	
		uint16_HSI[0] = 0;		
		mxSetData(HSI_out, uint16_HSI);	
		mxSetM(plhs[1], 1);
		mxSetN(plhs[1], 1);
		
		
		/*Timestamp just a 0*/
		plhs[2] = mxCreateNumericMatrix(1, 1, mxUINT64_CLASS, mxREAL);	
		timestamp_out = plhs[2];
		uint64 *timestamp;
		timestamp = static_cast<uint64 *> (mxMalloc(1*sizeof(double)));	
		timestamp[0] = 0;
		mxSetData(timestamp_out, timestamp);
		mxSetM(plhs[2], 1);
		mxSetN(plhs[2], 1);

		mxFree(HSI);
	}

	/*for(int i = 0; i < Nlam; i++) delete[] frames[i];*///CHECK
		return;
}


void getImage(unsigned char *HSI, mwSize *dimensions, double handle, int *wavelengths){

	int i_retCode;
    AT_H Hndl;
	Hndl = static_cast<AT_H> (handle);

    /*Frames is needed to keep the starting address of each frame*/
	unsigned char **frames;
	frames = new unsigned char* [dimensions[0]];

	/*for(int i = 0; i < Nlam; i++) {
		frames[i] = new unsigned char [i_imageSize];
	}*/
	
	frames[0] = HSI;
	int size;
	size = static_cast<int>(dimensions[3]);
    
    /*Queue first buffer, then queue all the others*/
	i_retCode = AT_QueueBuffer(Hndl, HSI, size);	
	
	int k=1;
	while (k < dimensions[0]){		
		frames[k] = &HSI[k*dimensions[3]];
		i_retCode = AT_QueueBuffer(Hndl, &HSI[k*dimensions[3]], dimensions[3]);	
		k++;
	}

    /*Set proper frame count*/
	AT_64 frameCount;
	frameCount = static_cast<AT_64> (dimensions[0]);
	i_retCode = AT_SetInt(Hndl, L"FrameCount", frameCount);
	i_retCode = AT_GetInt(Hndl, L"FrameCount", &frameCount);

    /*Set trigger mode*/
	int Index;
	AT_WC TriggerMode[256];
	i_retCode =	AT_SetEnumIndex(Hndl, L"TriggerMode", 4);
	i_retCode =	AT_GetEnumIndex(Hndl, L"TriggerMode", &Index);
	i_retCode =AT_GetEnumStringByIndex(Hndl, L"TriggerMode", Index, TriggerMode, 256);

	/*Check if camera is already acquiring*/
	AT_BOOL boole;
	i_retCode =AT_GetBool(Hndl, L"CameraAcquiring", &boole);
    
    /*Estimate waiting time: based on camera specifications*/
    double exp_time;
    AT_GetFloat(Hndl,L"ExposureTime", &exp_time);
    int waiting;
    waiting = static_cast<int> (1000*(exp_time+0.011));

    /*Start acquisition*/
	i_retCode = AT_Command (Hndl, L"AcquisitionStart");
		
	/* Set a clock to check performance*/
    clock_t trigger = clock();
		
    /*Send the software triggers*/
	for(int i = 0; i<dimensions[0]; i++){
        
		i_retCode = AT_Command(Hndl, L"SoftwareTrigger");	

		
		/* *
         * Here should be placed the code to 
         * communicate with the AOTF and select 
         * wavelength for each frame.
         * The last one is the Background Image
         * 
         */

		/*Sleep may be required to avoid losing Software triggers*/
        Sleep(waiting); 
	}	

	mexPrintf("\nTime trigger: %.9fs\n", (float)(clock() - trigger)/CLOCKS_PER_SEC);
    
    /*Set another clock to see which part is a bottle neck*/
	clock_t getImage = clock(); 
    
	unsigned char* WaitBuffer;
	int i_waitBufferSize;

	dimensions[5] = 1;
    
    /*Check that all the buffers have been written*/
	for (int j=0; j<dimensions[0]; j++){	

		i_retCode = AT_WaitBuffer(Hndl, &WaitBuffer, &i_waitBufferSize, 3000);	

 		if ( WaitBuffer != frames[j] ) {	
			dimensions[5] = 0;
			j = dimensions[0]-1;
		}
	}

    /*Stop acquisition free buffers*/
	i_retCode = AT_Command (Hndl, L"AcquisitionStop");
	i_retCode = AT_Flush(Hndl);

	
	delete[] frames;
	mexPrintf("\nTime getImage: %.9fs\n", (float)(clock() - getImage)/CLOCKS_PER_SEC);

}

uint64 extractMetadata(unsigned char* metadata, int length_metadata){

    /*Get the timestamps*/
	int *i_field, info[6], index, i_ticks[2];
	uint64 *ticks, timestamp;

	i_field = reinterpret_cast<int *>(metadata);
	index = length_metadata / 4;
	index--;
	for(int i=0; i<6; i++){
		info[i] = i_field[index-i];
		if(i==3 || i==2) i_ticks[i-2]=i_field[index-i];
	}

	ticks = reinterpret_cast<uint64 *>(i_ticks);
	timestamp = ticks[0];
	return timestamp;
}

