/*********************************************************************
* [double handle, double warning] = mOpenCamera(double *exposure_time);
*
* Output: it provides a handle to camera
*
* Open the camera, enable cooling and get the serial number.
********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include "atcore.h"
#include <iostream> 
#include "stdlib.h"

using namespace std;

double openCamera(mxArray *out_Handle, double *exposure_time);

void mexFunction(int nlhs, mxArray *plhs[], 
	int nrhs, const mxArray *prhs[])
{
	double *warning;

	mxArray *out_Handle, *warning_out;
	double *exposure_time;
    
    /* Get input*/
	exposure_time = mxGetPr(prhs[0]);
    
    /*Allocate memory for output*/
	warning = static_cast<double *> (mxMalloc(sizeof(double)));
    
    /*Prepare output*/
	plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
	out_Handle = plhs[0];

    /*Call openCamera (below)*/
	warning[0] = openCamera(out_Handle, exposure_time);

    /*output*/
	plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
	warning_out = plhs[1];
	mxSetPr(warning_out, warning);

}

double openCamera(mxArray *out_Handle, double *exposure_time){

	int i_retCode;
	double *d_Hndl;
	double warning=0;

    /*Initialise Library to control camera*/
	i_retCode = AT_InitialiseLibrary();
	if (i_retCode != AT_SUCCESS) {
		mexPrintf("WARNING! Error initialising library");
		warning = 1;
	}
	else {
		AT_64 iNumberDevices = 0;
		i_retCode =AT_GetInt(AT_HANDLE_SYSTEM, L"Device Count", &iNumberDevices);
		if (iNumberDevices <= 0) {
			mexPrintf("WARNING! No cameras detected");	
			warning = 1;
		}
		else {
			AT_H Hndl;
            /*Open camera and get handle*/
			i_retCode = AT_Open(0, &Hndl);    
            
            /*Allocate memory for output and assign value*/
			d_Hndl = static_cast<double *> (mxMalloc(sizeof(double)));	
			d_Hndl[0] = static_cast<double> (Hndl);//lo paso de AT_H a double					
			mxSetPr(out_Handle, d_Hndl);

			if (i_retCode != AT_SUCCESS) {
				mexPrintf("WARNING! Error getting handle");
				warning = 1;
			}
			else {
				mexPrintf("Successfully initialised camera: ");
                
                /*Set exposure time if camerra successfully intialised*/
				i_retCode = AT_SetFloat(Hndl, L"ExposureTime", *exposure_time);
                
                /*Check if it has been succesfully set*/
				double exp_time_applied;
				i_retCode = AT_GetFloat(Hndl, L"ExposureTime", &exp_time_applied);
				if (*exposure_time != exp_time_applied){
					warning = 1;
					mexPrintf("WARNING!! ");
				}

				/*Enable metadata*/
				i_retCode =	AT_SetBool(Hndl, L"MetadataEnable", 1);
				i_retCode =	AT_SetBool(Hndl, L"MetadataTimestamp", 1);

				/*Serial Number*/
				AT_WC serialnumber[64];
				i_retCode = AT_GetString(Hndl, L"Serial Number", serialnumber, 64);
				if (i_retCode == AT_SUCCESS) {

					char serial_number[64];							
					wcstombs(serial_number, serialnumber, 64);  // I have serial number as AT_WC and as char[64]
					mexPrintf("%s   ", serial_number);
				}
				else {
					mexPrintf("NO-SERIAL-NUM ");
					warning = 1;
				}
				mexPrintf("Exposure time set to: %f  ", exp_time_applied);

			}
		}
	}
	return warning;
}


