/*********************************************************************
* int resul = mSetLiveView(double handle);
     *input: handle to camera
     *output: resul contains if the operation has been succesfull or not

* It prepares the camera to start the Live View acquisition which
* means that frames will be acquired using a "light" function that does
* not do anything else 
********************************************************************/

#include <matrix.h>
#include <mex.h>   
#include "atcore.h"
#include <iostream> 
#include "stdlib.h"
using namespace std;


void mexFunction(int nlhs, mxArray *plhs[], 
	int nrhs, const mxArray *prhs[])
{
	double *handle, *d_retCode;
	mxArray *resul_out;
	int i_retCode;
    
    /*Get input*/
	handle = mxGetPr(prhs[0]);
    
    /*Allocate memory for output*/
	d_retCode = static_cast<double *> (mxMalloc(sizeof(d_retCode)));	

    /*Hndl must be AT_H when passed to camera*/
	AT_H Hndl;
	Hndl = static_cast<AT_H> (*handle);		

    /*Give a value to the output it may be changed later*/
	d_retCode[0] = 0; 
	
    /*Make sure that the camera frameCount limit is high enough*/
	AT_64 frameCount = 1000;
	i_retCode = AT_SetInt(Hndl, L"FrameCount", frameCount);
	i_retCode = AT_GetInt(Hndl, L"FrameCount", &frameCount);
    
    /*Disable Metadata as it won't be used*/
	i_retCode =	AT_SetBool(Hndl, L"MetadataEnable", 0);
	i_retCode =	AT_SetBool(Hndl, L"MetadataTimestamp", 0);

    /* Make sure camera is in Software trigger mode*/
	int Index;
	AT_WC TriggerMode[256];
	i_retCode =	AT_SetEnumIndex(Hndl, L"TriggerMode", 4);
	i_retCode =	AT_GetEnumIndex(Hndl, L"TriggerMode", &Index);
	i_retCode = AT_GetEnumStringByIndex(Hndl, L"TriggerMode", Index, TriggerMode, 256);

    /* Tell camera to start an acquisition*/
	i_retCode = AT_Command(Hndl, L"AcquisitionStart");

    /*Check success change output if necessary*/
	if ( i_retCode == AT_SUCCESS){
		d_retCode[0] = 1;
		mexPrintf("Acquisition running");
	}
	else
	{
		mexPrintf("WARNING!! ");
	}	                                                     
    /*Assign output*/
	plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
	resul_out = plhs[0];
	mxSetPr(resul_out, d_retCode);
}
